@@ -20,6 +20,8 @@
 #include "gtkdtree.h"
 #include "../vfs/vfs.h"
 
+#define FREEZE
+
 #ifdef HACK
 # define mc_opendir opendir
 # define mc_closedir closedir
@@ -35,13 +37,14 @@ enum {
 	DIRECTORY_CHANGED,
 	SCAN_BEGIN,
 	SCAN_END,
+	POSSIBLY_UNGRAB,
 	LAST_SIGNAL
 };
 
 static guint gtk_dtree_signals[LAST_SIGNAL] = { 0 };
 
 char *
-gtk_dtree_get_row_path (GtkDTree *dtree, GtkCTreeNode *row, gint column)
+gtk_dtree_get_row_path (GtkDTree *dtree, GtkCTreeNode *row)
 {
 	char *node_text, *path;
 
@@ -55,7 +58,7 @@ gtk_dtree_get_row_path (GtkDTree *dtree, GtkCTreeNode *row, gint column)
 		int val;
 
 		val = gtk_ctree_node_get_pixtext (
-			GTK_CTREE (dtree), row, column,
+			GTK_CTREE (dtree), row, 0,
 			&node_text, NULL, NULL, NULL);
 
 		if (!val)
@@ -190,7 +193,9 @@ static void
 scan_begin (GtkDTree *dtree)
 {
 	if (++dtree->scan_level == 1) {
+#ifdef FREEZE
 		gtk_clist_freeze (GTK_CLIST (dtree));
+#endif
 		gtk_signal_emit (GTK_OBJECT (dtree), gtk_dtree_signals[SCAN_BEGIN]);
 	}
 }
@@ -202,42 +207,43 @@ scan_end (GtkDTree *dtree)
 
 	if (--dtree->scan_level == 0) {
 		gtk_signal_emit (GTK_OBJECT (dtree), gtk_dtree_signals[SCAN_END]);
+#ifdef FREEZE
 		gtk_clist_thaw (GTK_CLIST (dtree));
+#endif
 	}
 }
 
 /* Scans a subdirectory in the tree */
 static void
-scan_subtree (GtkDTree *dtree, GtkCTreeNode *row)
+scan_subtree (GtkDTree *dtree, GtkCTreeNode *row, char *path)
 {
-	char *path;
-
 	dtree->loading_dir++;
-
 	scan_begin (dtree);
-	path = gtk_dtree_get_row_path (dtree, row, 0);
-
-	if (dtree->current_path)
-		g_free (dtree->current_path);
-
-	dtree->current_path = path;
 
 	gtk_dtree_load_path (dtree, path, row, 1);
 
-	dtree->loading_dir--;
 	scan_end (dtree);
+	dtree->loading_dir--;
 }
 
 static void
 gtk_dtree_select_row (GtkCTree *ctree, GtkCTreeNode *row, gint column)
 {
 	GtkDTree *dtree;
+	char *path;
 
 	dtree = GTK_DTREE (ctree);
 
 	if (dtree->removing_rows)
 		return;
 
+	/* Ask for someone to ungrab the mouse, as the stupid clist grabs it on
+	 * button press.  We cannot do it unconditionally because we don't want
+	 * to knock off a DnD grab.  We cannot do it in a button_press handler,
+	 * either, because the row is selected *inside* the default handler.
+	 */
+	gtk_signal_emit (GTK_OBJECT (dtree), gtk_dtree_signals[POSSIBLY_UNGRAB], NULL);
+
 	scan_begin (dtree);
 
 	(* parent_class->tree_select_row) (ctree, row, column);
@@ -249,11 +255,19 @@ gtk_dtree_select_row (GtkCTree *ctree, GtkCTreeNode *row, gint column)
 
 	dtree->last_node = row;
 
-	scan_subtree (dtree, row);
+	/* Set the new current path */
+
+	path = gtk_dtree_get_row_path (dtree, row);
+
+	if (dtree->current_path)
+		g_free (dtree->current_path);
+
+	dtree->current_path = path;
+
+	scan_subtree (dtree, row, path);
 
 	if (!dtree->internal)
-		gtk_signal_emit (GTK_OBJECT (dtree), gtk_dtree_signals[DIRECTORY_CHANGED],
-				 dtree->current_path);
+		gtk_signal_emit (GTK_OBJECT (dtree), gtk_dtree_signals[DIRECTORY_CHANGED], path);
 
 	scan_end (dtree);
 }
@@ -299,7 +313,7 @@ gtk_dtree_do_select_dir (GtkDTree *dtree, char *path)
 	if (dtree->current_path && (strcmp (path, dtree->current_path) == 0))
 		return TRUE;
 
-	s = alloca (strlen (path)+1);
+	s = alloca (strlen (path) + 1);
 	strcpy (s, path);
 	current_node = dtree->root_node;
 
@@ -390,7 +404,8 @@ gtk_dtree_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
 	char *request;
 
 	GTK_WIDGET_CLASS (parent_class)->size_allocate (widget, allocation);
-	if (allocation->width != 0 && allocation->height != 0)
+
+	if (allocation->width > 1 && allocation->height > 1)
 		dtree->visible = TRUE;
 	else
 		dtree->visible = FALSE;
@@ -421,12 +436,18 @@ static void
 gtk_dtree_expand (GtkCTree *ctree, GtkCTreeNode *node)
 {
 	GtkDTree *dtree;
+	char *path;
 
 	dtree = GTK_DTREE (ctree);
 
 	scan_begin (dtree);
+
 	(* parent_class->tree_expand) (ctree, node);
-	scan_subtree (dtree, node);
+
+	path = gtk_dtree_get_row_path (dtree, node);
+	scan_subtree (dtree, node, path);
+	g_free (path);
+
 	scan_end (dtree);
 }
 
@@ -540,10 +561,12 @@ tree_set_freeze (int state, void *data)
 {
 	GtkDTree *dtree = GTK_DTREE (data);
 
+#ifdef FREEZE
 	if (state)
 		gtk_clist_freeze (GTK_CLIST (dtree));
 	else
 		gtk_clist_thaw (GTK_CLIST (dtree));
+#endif
 }
 
 static void
@@ -600,16 +623,23 @@ gtk_dtree_class_init (GtkDTreeClass *klass)
 				gtk_marshal_NONE__NONE,
 				GTK_TYPE_NONE,
 				0);
+	gtk_dtree_signals[POSSIBLY_UNGRAB] =
+		gtk_signal_new ("possibly_ungrab",
+				GTK_RUN_FIRST, object_class->type,
+				GTK_SIGNAL_OFFSET (GtkDTreeClass, possibly_ungrab),
+				gtk_marshal_NONE__NONE,
+				GTK_TYPE_NONE,
+				0);
 
 	gtk_object_class_add_signals (object_class, gtk_dtree_signals, LAST_SIGNAL);
 
-	object_class->destroy        = gtk_dtree_destroy;
+	object_class->destroy = gtk_dtree_destroy;
 
-	widget_class->size_allocate  = gtk_dtree_size_allocate;
+	widget_class->size_allocate = gtk_dtree_size_allocate;
 
 	ctree_class->tree_select_row = gtk_dtree_select_row;
-	ctree_class->tree_expand     = gtk_dtree_expand;
-	ctree_class->tree_collapse   = gtk_dtree_collapse;
+	ctree_class->tree_expand = gtk_dtree_expand;
+	ctree_class->tree_collapse = gtk_dtree_collapse;
 }
 
 static void
@@ -645,7 +675,6 @@ gtk_dtree_load_root_tree (GtkDTree *dtree)
 
 	/* Select root node */
 	gtk_ctree_select (GTK_CTREE (dtree), dtree->root_node);
-
 	gtk_clist_thaw (GTK_CLIST (dtree));
 }
 
