@@ -30,7 +30,7 @@
 #include "win.h"
 #include "key.h"	/* For mi_getch() */
 
-/* "$Id: menu.c,v 1.16 1999/06/24 06:47:03 norbert Exp $" */
+/* "$Id: menu.c,v 1.19 1999/08/08 12:43:37 norbert Exp $" */
 
 extern int is_right;
 int menubar_visible = 1;	/* This is the new default */
@@ -50,25 +50,33 @@ static void menu_scan_hotkey(Menu menu)
 Menu create_menu (char *name, menu_entry *entries, int count)
 {
     Menu menu;
+    char *cp;
 
     menu = (Menu) g_malloc (sizeof (*menu));
     menu->count = count;
-    menu->max_entry_len = 0;
+    menu->max_entry_len = 20;
     menu->entries = entries;
 
+    if (entries != (menu_entry*) NULL) {
+	register menu_entry* mp;
+	for (mp = entries; count--; mp++) {
+	    if (mp->text[0] != '\0') {
 #ifdef ENABLE_NLS
-	if (entries != (menu_entry*) NULL)
-	{
-		register menu_entry* mp;
-		for (mp = entries; count--; mp++)
-		{
-			if (mp->text[0] == '\0')
-				continue;
-
-			mp->text = _(mp->text);
-		}
-	}
+	        mp->text = _(mp->text);
 #endif /* ENABLE_NLS */
+	        cp = strchr (mp->text,'&');
+
+		if (cp != NULL && *(cp+1) != '\0') {
+		    mp->hot_key = tolower (*(cp+1));
+		    menu->max_entry_len = max (strlen (mp->text) - 1,
+			menu->max_entry_len);
+		} else {
+		    menu->max_entry_len = max (strlen (mp->text),
+			menu->max_entry_len);
+		}
+	    }
+	}
+    }
 
     menu->name = g_strdup ( _(name) );
     menu_scan_hotkey(menu);
@@ -78,13 +86,7 @@ Menu create_menu (char *name, menu_entry *entries, int count)
 
 static void menubar_drop_compute (WMenu *menubar)
 {
-    const Menu menu = menubar->menu [menubar->selected];
-    int   max_entry_len = 0;
-    int i;
-
-    for (i = 0; i < menu->count; i++)
-	max_entry_len = max (max_entry_len, strlen (menu->entries [i].text));
-    menubar->max_entry_len = max_entry_len = max (max_entry_len, 20);
+    menubar->max_entry_len = menubar->menu [menubar->selected]->max_entry_len;
 }
 
 static void menubar_paint_idx (WMenu *menubar, int idx, int color)
@@ -99,27 +101,24 @@ static void menubar_paint_idx (WMenu *menubar, int idx, int color)
     widget_move (&menubar->widget, y, x);
     attrset (color);
     hline (' ', menubar->max_entry_len+2);
-    if (!*menu->entries [idx].text){
+    if (!*menu->entries [idx].text) {
     	attrset (SELECTED_COLOR);
         widget_move (&menubar->widget, y, x + 1);
     	hline (slow_terminal ? ' ' : ACS_HLINE, menubar->max_entry_len);
     } else {
-	unsigned char *text = menu->entries [idx].text;
+	unsigned char *text;
 
 	addch((unsigned char)menu->entries [idx].first_letter);
 	for (text = menu->entries [idx].text; *text; text++)
 	{
-		if (*text == '&')
-		{
-			++text;
-			menu->entries [idx].hot_key = tolower(*text);
-			attrset (color == MENU_SELECTED_COLOR ?
-				MENU_HOTSEL_COLOR : MENU_HOT_COLOR);
-			addch(*text);
-			attrset(color);
-			continue;
+		if (*text != '&')
+		    addch(*text);
+		else {
+		    attrset (color == MENU_SELECTED_COLOR ?
+			MENU_HOTSEL_COLOR : MENU_HOT_COLOR);
+		    addch(*(++text));
+		    attrset(color);
 		}
-		addch(*text);
 	}
     }
     widget_move (&menubar->widget, y, x + 1);
